import { parser } from 'lezer-json';
import { LezerSyntax, indentNodeProp, continuedIndent, foldNodeProp } from '@codemirror/next/syntax';
import { styleTags } from '@codemirror/next/highlight';

const jsonSyntax = LezerSyntax.define(parser.withProps(indentNodeProp.add({
    Object: continuedIndent({ except: /^\s*\}/ }),
    Array: continuedIndent({ except: /^\s*\]/ })
}), foldNodeProp.add({
    Object(subtree) { return { from: subtree.start + 1, to: subtree.end - 1 }; },
    Array(subtree) { return { from: subtree.start + 1, to: subtree.end - 1 }; }
}), styleTags({
    String: "string",
    Number: "number",
    "True False": "bool",
    PropertyName: "propertyName",
    null: "null",
    ",": "separator",
    "[ ]": "squareBracket",
    "{ }": "brace"
})), {
    languageData: {
        closeBrackets: { brackets: ["[", "{", '"'] },
        indentOnInput: /^\s*[\}\]]$/
    }
});
/// Returns an extension that installs the JSON syntax provider.
function json() {
    return jsonSyntax;
}

export { json, jsonSyntax };
